// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once

/// Child dialog that contains a set of controls for adjusting the display attributes
/// for a single primitive object.
class CPrimitiveControl : public CDialog
{
public:
	CPrimitiveControl(CWnd* pParent = NULL);   // standard constructor

  /// The IDD enum is a common technique in MFC to associate a dialog with
  /// a resource that you edit in the dialog editor
  enum { IDD = IDD_PRIMITIVE_CONTROL };

  /// Set the primitive that this control works with. The primitive must not
  /// be deleted or moved in memory while this control maintains a pointer to it.
  /// If a primitive is going to be deleted, call SetPrimitive( NULL ) on this control
  /// first.
  void SetPrimitive(UWB_Primitive *p);

  /// Update all of the controls to match the state of the current primitive.
  /// If no primitive is active, the controls are hidden
  void PrimitiveSetsControl();

protected:
  /// The virtual DoDataExchange is an MFC method for synchronizing values in this
  /// class with their corresponding controls on the dialog.
  /// See MSDN documentation for more information
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

  /// Called immediately after a dialog is created, but before it is displayed.
  /// This is where all of the control initialization usually occurs. This is
  /// also where the graphics window is set up
  virtual BOOL OnInitDialog();

  /// A message map is an MFC macro for mapping window's events (paint, size, mouse,...) that
  /// occur on a window to functions in this class.
  /// The functions in this class that begin with afx_msg are "handlers" for window's messages
  DECLARE_MESSAGE_MAP()

  /// This message is sent before a control is drawn. Return a brush that will be used to draw
  /// the background of the control. This is used to colorize the red, green, and blue labels
  afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);

  ///
  /// Called when the slider states change through user input.
  afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

  /// button click message handlers
  afx_msg void OnBnClickedTexture();
  afx_msg void OnBnClickedVisibility();
  afx_msg void OnBnClickedShading();
  afx_msg void OnBnClickedBlending();
  afx_msg void OnBnClickedWireFrame();
  afx_msg void OnBnClickedBoundingVol();
  afx_msg void OnBnClickedLighting();
  afx_msg void OnBnClickedObjectColor();
  afx_msg void OnBnClickedShadeColor();
  afx_msg void OnBnClickedDiffuseColor();
  afx_msg void OnBnClickedSpecularColor();
  afx_msg void OnBnClickedShiningness();
  afx_msg void OnBnClickedBrowseTexturefile();

private:
  enum ColorSlidersControl 
  {
    eSliderControlObjColor = 0,
    eSliderControlShadeColor = 1,
    eSliderControlDiffuseColor = 2,
    eSliderControlSpecularColor = 3,
    eSliderControlShiningness = 4
  };

  UWBMFC_SliderCtrlWithEcho m_TextureIndex;
  void ColorSlidersSetPrimitive();
  void PrimitiveSetsColorSliders();
  void InitASliderToColor();
  void InitASliderToN();

  ColorSlidersControl m_SliderControl;
  UWB_Primitive *m_Primitive;

  UWBMFC_SliderCtrlWithEcho m_ColorR;
  UWBMFC_SliderCtrlWithEcho m_ColorG;
  UWBMFC_SliderCtrlWithEcho m_ColorB;
  UWBMFC_SliderCtrlWithEcho m_ColorA;
  BOOL m_Texture;
  BOOL m_Visibility;
  BOOL m_Shading;
  BOOL m_Blending;
  BOOL m_WireFrame;
  BOOL m_BoundingVol;
  BOOL m_Lighting;
  CString m_TextureFileEcho;
};
