// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "resource.h"
#include "afxcmn.h"
#include "XformInfoControl.h"
#include "PrimitiveControl.h"

/// Child dialog that contains a large set of controls for adjusting the
/// contents of a scenenode data structure
class CSceneTreeControl : public CDialog
{
public:
  CSceneTreeControl(CWnd* pParent = NULL);   // standard constructor

  /// The IDD enum is a common technique in MFC to associate a dialog with
  /// a resource that you edit in the dialog editor
  enum { IDD = IDD_SCENETREE_CONTROL };

protected:
  /// The virtual DoDataExchange is an MFC method for synchronizing values in this
  /// class with their corresponding controls on the dialog.
  /// See MSDN documentation for more information
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

  /// Called immediately after a dialog is created, but before it is displayed.
  /// This is where all of the control initialization usually occurs. This is
  /// also where the graphics window is set up
  virtual BOOL OnInitDialog();

  /// A message map is an MFC macro for mapping window's events (paint, size, mouse,...) that
  /// occur on a window to functions in this class.
  /// The functions in this class that begin with afx_msg are "handlers" for window's messages
  DECLARE_MESSAGE_MAP()

  /// Called when the user changes the selection state of the tree control
  /// changes.
  afx_msg void OnTvnSelchangedSceneTree(NMHDR *pNMHDR, LRESULT *pResult);

  ///
  /// Called when the slider states change through user input.
  afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

private:
  /// Tree control that displays the hierarchy of a scene node
  /// data structure
  CTreeCtrl m_SceneTree;

  /// Transformation controls for a primitive
  /// See XformInfoControl.h
  CXformInfoControl m_XformControl;

  /// Controls for adjusting a primitives display attributes
  /// See PrimitiveControl.h
  CPrimitiveControl m_PrimitiveControl;

  /// If the current scenenode contains a primitive list, m_CurrentList is
  /// set to this list. The slider and echo control which item in the list
  /// is adjusted by the attribute controls in the CPrimitiveControl set
  UWB_PrimitiveList *m_CurrentList;
  CString m_PrimEcho;
  CSliderCtrl m_PrimSelect;
};