#include "stdafx.h"
#include "TutorialApp.h"
#include "TutorialDlg.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif


CTutorialDlg::CTutorialDlg(CWnd* pParent)
: CDialog(CTutorialDlg::IDD, pParent)
, m_ShowLgtCtrl(FALSE)
, m_ShowMeshCtrl(FALSE)
{
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTutorialDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Check(pDX, IDC_SHOWLIGHT_CONTROL, m_ShowLgtCtrl);
  DDX_Check(pDX, IDC_SHOWMESH_CONTROL, m_ShowMeshCtrl);
  DDX_Control(pDX, IDC_Z, m_Z);
}

BEGIN_MESSAGE_MAP(CTutorialDlg, CDialog)
  ON_WM_PAINT()
  ON_WM_DESTROY()
  ON_WM_TIMER()
  ON_BN_CLICKED(IDC_SHOWLIGHT_CONTROL, OnBnClickedLightControl)
  ON_BN_CLICKED(IDC_SHOWMESH_CONTROL, OnBnClickedMeshControl)
END_MESSAGE_MAP()

// This is called when the dialog is first created and shown.
// It is a good spot to initialize member variables.
BOOL CTutorialDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);   // Set big icon
  SetIcon(m_hIcon, FALSE);  // Set small icon

  if( !UWBD3D_GraphicsSystem::GetSystem().CreateGraphicsContext(m_hWnd) )
    return FALSE;

  if( !m_MainView.Initialize(*this, IDC_GRFX_PLACEHOLDER) )
    return FALSE;
  m_MainView.GetCamera().SetCameraLookAt(0, 0, 0);
  m_MainView.GetCamera().SetCameraPosition(0, 10, -25);
  m_MainView.GetCamera().SetViewFrustumInDegree(45, 0.5f, 100);

  if (!m_SmallView.Initialize(*this, IDC_SMALLVIEW_PLACEHOLDER) )
    return FALSE;
  m_SmallView.GetCamera().SetCameraLookAt(0, 0, 0);
  m_SmallView.GetCamera().SetCameraPosition(30, 30, 30);
  m_SmallView.GetCamera().SetViewFrustumInDegree(45, 0.5f, 150);

  m_Z.Initialize(0.7f, 1.0f, 0.7f);


  if (!UWBMFC_ReplaceDialogControl(*this, IDC_PLACEHOLDER, m_TreeControl, CSceneTreeControl::IDD) )
    return FALSE;

  m_LightControl.Create(IDD_LIGHT_CONTROL, this);
  m_LightControl.ShowWindow(SW_HIDE);

  m_MeshControl.Create(IDD_INTERACTIVE_MESH_CONTROL, this);
  m_MeshControl.ShowWindow(SW_HIDE);
  m_MeshControl.SetCustomMesh( theApp.GetModel().GetCustomMesh() );

  CWnd* pWnd = GetDlgItem(IDC_STATUS);
  if (NULL != pWnd)
    UWB_hEchoWindow = pWnd->m_hWnd;
  UWB_EchoToStatusArea(L"Application Initialization Completed");

  // Here we initialize the timer, to go off every 25 milli seconds
  // when timer "goes-off", our OnTimer() function will be called and
  // it is up to us to decide what we want to do.
  int msInterval = 25; // timer going off every 25 milli seconds.
  SetTimer(0, msInterval, NULL);    // our goal is to approx 40 frames per second

  UWB_EchoToStatusArea( L"OnInitDialog Complete" );

  return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CTutorialDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this); // device context for painting

    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;

    // Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {
    CDialog::OnPaint();
  }
}

void CTutorialDlg::OnDestroy()
{
  UWBD3D_GraphicsSystem::GetSystem().ShutDown();
  CDialog::OnDestroy();
}

void CTutorialDlg::OnTimer(UINT nIDEvent)
{
  theApp.GetModel().UpdateSimulation();
  //Draw the view that shows our model

  m_MainView.SetZ(m_Z.GetSliderValue() );
  m_SmallView.SetZ( m_Z.GetSliderValue() );
  m_MainView.DrawGraphics();
  m_SmallView.DrawGraphics();
}

void CTutorialDlg::OnBnClickedLightControl()
{
  UpdateData(TRUE);
  if (m_ShowLgtCtrl)
     m_LightControl.ShowWindow(SW_SHOW);
  else
    m_LightControl.ShowWindow(SW_HIDE);
}

void CTutorialDlg::OnBnClickedMeshControl()
{
  UpdateData(TRUE);
  if (m_ShowMeshCtrl)
      m_MeshControl.ShowWindow(SW_SHOW);
  else
      m_MeshControl.ShowWindow(SW_HIDE);
}
