#include "uwbgl_Camera3.h"

void UWB_Camera::GetMouseRotationsInRadian(float &xRot, vec3 &xRef, float &yRot, vec3 &yRef) const 
{
  vec3 view_direction;
  yRef =  m_UpVector;
  view_direction = m_LookAt - m_CameraPos;
  xRef = cross(m_UpVector, view_direction);
  normalize(xRef);
  yRef.x = 0.0f;
  yRef.y = 1.0f;
  yRef.z = 0.0f;  // sense of up is important.

  xRot = UWB_ToRadians(m_XRot);
  yRot = UWB_ToRadians(m_YRot);
}

void UWB_Camera::ResetRotation()
{
  m_XRot = 0.0f;
  m_YRot = 0.0f;
}

void UWB_Camera::BeginMouseRotate(int x, int y)
{
  m_XRotStart = m_XRot;
  m_YRotStart = m_YRot;
  m_TmpInitXPos = x;
  m_TmpInitYPos = y;
}

void UWB_Camera::MouseRotate(int x, int y)
{
  m_XRot = m_XRotStart + (y-m_TmpInitYPos);   // says each pixel movement is mapped to one degree
  m_YRot = m_YRotStart + (x-m_TmpInitXPos);
}


void UWB_Camera::BeginMouseTrack(int x, int y)
{
  m_TmpInitXPos = x;
  m_TmpInitYPos = y;
}

#define TRACK_FACTOR 0.1f

void UWB_Camera::MouseTrack(int x, int y)
{
  float dx = float(x-m_TmpInitXPos) * TRACK_FACTOR;     // each pixel tracks 0.2 WC unit
  float dy = float(y-m_TmpInitYPos) * TRACK_FACTOR;

  vec3 view_direction, cross_direction;
  vec3 up_direction = m_UpVector;
  view_direction = m_LookAt - m_CameraPos;

  cross_direction = cross(up_direction, view_direction);
  normalize(cross_direction);

  up_direction = cross(view_direction, cross_direction);
  normalize(up_direction);

  vec3 delta;
  delta = (-dx * cross_direction) + (dy * up_direction);
  m_CameraPos = m_CameraPos - delta;
  m_LookAt = m_LookAt - delta;

  // continue to track
  BeginMouseTrack(x, y);
}

void UWB_Camera::BeginMouseZoom(int x)
{
  m_TmpInitXPos = x;
}

void UWB_Camera::MouseZoom(int x)
{
  float dx = float(x - m_TmpInitXPos) * TRACK_FACTOR;
  vec3 view_vec = m_LookAt - m_CameraPos;
  float view_distance = length(view_vec);
  normalize(view_vec);
  m_CameraPos = m_LookAt - ( (view_distance + dx) * view_vec);
  BeginMouseZoom(x);
}