#include "uwbgl_D3DPrimitiveMeshCustom1.h"
#include "uwbgl_PrimitiveLine6.h"
#include "uwbgl_PrimitivePoint6.h"

void UWBD3D_PrimitiveMeshCustom::DrawPrimitive(eLevelOfDetail lod, UWB_DrawHelper& dh)
{
  if ((m_Initialized) || (InitializeMesh())) {
	    //
	    // 1. make sure we load the new value of the selected vertex into the hardware ...
	    //
	    SetSelectedVertexToD3D();

	    //
	    // 2. Recompute Normal: this is VERY EXPENSIVE, should optimize in real application ...
	    D3DXComputeNormals(m_Mesh, NULL);

      __super::DrawPrimitive(lod, dh);   // this will draw the entire mesh

      if (DrawControlGridIsEnabled())
            DrawMeshControlGrid(lod, dh);

      if (DrawNormalIsEnabled())
            DrawNormalVectors(lod, dh);
  }
}

void 
UWBD3D_PrimitiveMeshCustom::DrawNormalVectors(eLevelOfDetail lod, UWB_DrawHelper& dh)
{
  UWB_PrimitiveLine l;
  l.EnableLighting(false);
  l.SetShadeMode(smFlat);
  l.SetFlatColor(UWB_Color(1.0f, 1.0f, 1.0f, 0.0f));

  void *pVBuffer;
	m_Mesh->LockVertexBuffer( 0, &pVBuffer );
			DeviceVertexFormat *v = (DeviceVertexFormat *)pVBuffer;
			for (int i=0; i<m_NumVertices; i++) {
          D3DXVECTOR3 n = v[i].m_Normal;
					D3DXVECTOR3 p = v[i].m_Point + (n * 0.1f);
          l.SetStartPoint(v[i].m_Point.x, v[i].m_Point.y, v[i].m_Point.z);
					l.SetEndPoint(p.x, p.y, p.z);
					l.Draw(lod, dh);
		}
	m_Mesh->UnlockVertexBuffer();
}


void
UWBD3D_PrimitiveMeshCustom::DrawMeshControlGrid(eLevelOfDetail lod, UWB_DrawHelper& dh)
{
	UWB_PrimitivePoint sph;
  sph.EnableLighting(false);
  sph.SetShadeMode(smFlat);
  sph.SetFlatColor(UWB_Color(0.8f, 0.8f, 0.8f, 0.0f));
  sph.SetPointSize(10.0f);

	void *pVBuffer;
	m_Mesh->LockVertexBuffer( 0, &pVBuffer );
		DeviceVertexFormat *v = (DeviceVertexFormat *)pVBuffer;
		for (int i=0; i<m_NumVertices; i++) {
				sph.SetLocation(v[i].m_Point.x, v[i].m_Point.y, v[i].m_Point.z);
				sph.Draw(lod, dh);
		}
	m_Mesh->UnlockVertexBuffer();

	//
	// Now draw the selected vertex ...
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		sph.SetFlatColor(UWB_Color(1.0f, 0.0f, 0.0f, 0.0f));
		sph.SetPointSize(15.0f);
		sph.SetLocation(m_SelectedVertex.x, m_SelectedVertex.y, m_SelectedVertex.z);
		sph.Draw(lod, dh);
	}

}